#ifndef READERENGINE_H
#define READERENGINE_H

#include <QtConcurrentRun>

#include "FTDIDevice.h"
#include "SIPPProtocol.h"
#include "OffsetControl.h"
#include "DefaultDataProducer.h"
#include "StatusData.h"

/*!
 * \class ReaderEngine ReaderEngine.h
 *
 * \brief It performs the operations for controlling the streaming from the
 * hardware device. It is in charge of setting up the hardware that allows to
 * communicate with an Enobio device in order to receive the generated the data
 * stream.
 */
class ReaderEngine
{
public:

    // ---------------------Public variables---------------------

    /*!
     * The number of channels to read. This is accesible statically from other
     * classes.
     */
    static const int NUMBER_OF_CHANNELS;



    // ----------------------Public methods----------------------

    /*!
     * Constructor that gets both hardware device and protocol in order to
     * access the ECG/EEG data streaming.
     *
     * \param hwDevice Reference to a FTDIDevice object in order to perform the
     * hardware reading and writings.
     *
     * \param protocol Protocol object that allows to parse the received data
     * protocol.
     */
    ReaderEngine (FTDIDevice& hwDevice, SIPPProtocol& protocol);

    /*!
     * Empty destructor
     */
    ~ReaderEngine ();

    /*!
     * It performs the operations required to initialize the hardware
     */
    bool open ();

    /*!
     * It performs the operations required to close the hardware device
     */
    void close ();

    /*!
     * Start sampling at the predefined sampling rate.
     *
     * \return True or false according to the success
     * or failure of the operation
     */
    bool start ();

    /*!
     * Hold sampling (setting sampling rate to 0)
     *
     * \return True or false according to the success
     * or failure of the operation
     */
    bool stop ();

    /*!
     * It checks if the recorded device is present.
     */
    bool ping ();

    /*!
     * It returns the Property object identified with a property name.
     *
     * \param propertyName Name of the property to be obtained.
     *
     * \return Reference to the solicited property.
     */
    const Property& getProperty (std::string propertyName);

    /*!
     * It configures an API property to be identical to the parameter.
     *
     * \param property Configuration property object.
     *
     * \return It returns true if the method succeeded on setting the property
     * Otherwise a null object is returned.
     */
    bool setProperty(const Property& property);

    /*!
     * Access to the data producer in charge of populating the ECG/EEG data
     *
     * \return Constant reference to the instance of the ECG/EEG streaming data
     * producer.
     */
    IDataProducer& getStreamingDataProducer ();

    /*!
     * Access to the data producer in charge of populating the status of the
     * ECG/EEG channels and the device itself.
     *
     * \return Constant reference to the instance of the status data producer
     */
    IDataProducer& getStatusDataProducer ();

    /*!
     * It adds a DataConsumer to the list of consumers of the local streaming
     * data producer
     *
     * \param dataConsumer The DataConsumer to add
     */
    void addStreamingDataConsumerEndPoint ( IDataConsumer& dataConsumer);

    /*!
     * It adds a DataConsumer to the list of consumers of the status
     * data producer of ReaderEngine
     *
     * \param dataConsumer The DataConsumer to add
     */
    void addStatusDataConsumerEndPoint ( IDataConsumer& dataConsumer);

    /*!
     * It removes a DataConsumer from the list of consumers of the streaming
     * data producer of ReaderEngine
     *
     * \param dataConsumer The DataConsumer to remove
     */
    void removeStreamingDataConsumerEndPoint
                                           (const IDataConsumer& dataConsumer);

    /*!
     * It removes a DataConsumer from the list of consumers of the local status
     * data producer
     *
     * \param dataConsumer The DataConsumer to remove
     */
    void removeStatusDataConsumerEndPoint (const IDataConsumer& dataConsumer);

    /*!
     * Set the USB transmission channel
     *
     * \param channel The channel to set
     * \return True or false according to whether the command was sent correctly
     */
    bool setUSBTransmissionChannel(unsigned char channel);

    /*!
     * Set the Enobio transmission channel
     *
     * \param channel The channel to set
     * \return True or false according to whether the command was sent correctly
     */
    bool setEnobioTransmissionChannel(unsigned char channel);

    /*!
     * Get the USB transmission channel
     *
     * \return The current transmission channel of the USB radio
     */
    char getUSBTransmissionChannel();

    /*!
     * Get the Enobio transmission channel
     *
     * \return The current transmission channel set in the Enobio radio
     */
    unsigned char getEnobioTransmissionChannel();

    /*!
     * Get the USB firmware version. This is just an integer that to a real
     * version needs to passed through a look up table to give the true value
     *
     * \return The firmware version
     */
    unsigned char getUSBFWVersion();

    /*!
     * Get the Enobio firmware version. This is just an integer that to a real
     * version needs to passed through a look up table to give the true value
     *
     * \return The firmware version
     */
    unsigned char getEnobioFWVersion();

    /*!
     * Enable or disable the motitoring of the values of a channel. If monitored
     * then calibration of the offset value will take place every time the
     * channel values are out or a specific range.
     *
     * \param index The index of the channel to calibrate (0,1,2,3)
     * \return It will return true if the parameter status is either 0 or 1.
     * False otherwise
     */
    bool setCalibrateChannel(unsigned char index, unsigned char status);

    /*!
     * Check whether a channel is being monitored or not.
     *
     * \param index The index of the channel to check (0,1,2,3)
     * \return True or false according to whether the channel is calibrated
     * or not.
     */
    bool getCalibrateChannel(unsigned char index);


private:


    // ---------------------Private variables---------------------

    /*!
     * This used to control the inner thread of the class. This thread is
     * responsible for the reading of the data.
     */
    QFuture<bool> _threadResult;

    /*!
     * This is the timestamp of the creation of each sample.
     */
    qint64 _timestamp;

    /*!
     * This is a temporary array to hold the previous status of a channel
     * so that a change of status is detected.
     */
    StatusData::statusCode * _status;

    /*!
     * Instance of the ECG/EEG channel offset corrector.
     */
    OffsetControl _offsetControl;

    /*!
     * Instance of the data producer that streams the ECG/EEG data.
     */
    DefaultDataProducer _streamingDataProducer;

    /*!
     * Instance of the data producer that populates the status of the ECG/EEG
     * data channels as well as the status device itself.
     */
    DefaultDataProducer _statusDataProducer;

    /*!
     * Reference to the hardware device
     */
    FTDIDevice& _hwDevice;

    /*!
     * Reference to the protocol object that is being used
     */
    SIPPProtocol& _protocol;

    /*!
     * This is a boolean to control the reading loop. When set to false
     * the operation of the thread is terminated.
     */
    bool _isReading;

    /*!
     * Temporary counter
     */
    unsigned char _counter;

    /*!
     * A flag that is used only when an offset change has happened
     */
    unsigned char _offsetFlag;

    /*!
     * This shows whether we are meant to have streaming or not.
     */
    bool _streaming;

    /*!
      * Mutex that synchronizes start/stop streaming within the read thread and
      * outside it.
      */
    QMutex streamingMutex;

    /*!
      * Mutex that synchronizes read/write to the ftdi device.
      */
    QMutex readWriteMutex;


    //---------------------Private methods---------------------

    /*!
     * To Be Commented
     */
    bool _getUSBId ();

    /*!
     * Request or set the USB or Enobio transmission channel
     *
     * \param channel The value to set as the channel. If the boolean read is
     * false this parameter is ignored.
     * \param local If true the method sends to the USB radio,
     * if false it sends to the Enobio radio
     * \param read If true it will request the channel, if false it will set it

     */
    bool _sendTransmissionChannel(unsigned char channel, bool local, bool read);

    /*!
     * Request the local USB ID in order to setup the communication with Enobio
     *
     * \return True or false depending on if the command was sent correctly
     */
    bool _requestID ();

    /*!
     * Request the USB or Enobio firmware version
     *
     * \param local If true the request is sent to the USB radio, if false it
     * is sent to the Enobio one
     *
     * \return True or false depending on if the command was sent correctly
     */
    bool _sendVersion(bool local);

    /*!
     * Check the previous state of a channel and if different with the
     * current one then notify the status consumers
     *
     * \param flag The current calibration status
     */
    void _notifyCalibrationStatus(char flag);

    /*!
     * Start waiting for incoming data. This method will run in a seperate
     * thread.
     *
     * \return True or false according to success or failure due to en error
     */
    bool _read ();

    /*!
     * Function used to flush the output buffer just before sending
     * a SIPP packet
     *
     * \param buffer The bytes to send
     * \param numBytes The size of the buffer array.
     */
    void _writeCommand(unsigned char * buffer, unsigned long numBytes);

    /*!
     * This method constucts the structure of an Enobio sample to be sent
     * to consumers
     *
     * \param enobioBytes A buffer with 8 bytes making up an Enobio data sample
     * \param flag This indicates whether the data is real or it has been
     * replicated because of a lost packet.
     * \param offsetSample If a change of offset has happened, then this
     * parameter will define the number of samples before the current one, the
     * change has happened.
     */
    void _processSample (const unsigned char * enobioBytes, unsigned char flag,
                       unsigned short offsetSample);

    /*!
     * This method is called every time data is available. It will request the
     * available data and process it sample by sample
     */
    void _processData ();

    /*!
     * This will reset variables everytime the streaming is stopped.
     */
    void _reset ();

     /*!
      * Send the command to change the offset of a specific channel
      *
      * \return True or false depending on if the command was sent correctly
      */
     bool _sendOffset ();

     /*!
      * Send the command to start the data streaming
      *
      * \return True or false depending on if the command was sent correctly
      */
     bool _sendStart ();

     /*!
      * Send the command to stop the data streaming
      *
      * \return True or false depending on if the command was sent correctly
      */
     bool _sendStop ();

     /*!
      * Translate a digital value to the equivalent voltage
      *
      * \param voltage The digital value (0-65536)
      *
      * \return A voltage value (0- 38570 uV)
      */
     int _getVolts (int voltage);

     /*!
      * This creates a PData object with an status message encapsulated
      * and sends it to any registered consumers.
      *
      * \param code The code of the StatusData object to be send
      */
     void _notifyStatus (StatusData::statusCode code);

     /*!
      * This creates a PData object with an Enobio data sample encapsulated
      * and sends it to any registered consumers.
      *
      * \param dataCh1 Value for channel 1
      * \param dataCh2 Value for channel 2
      * \param dataCh3 Value for channel 3
      * \param dataCh4 Value for channel 4
      * \param flag This can be:
      * 0: Real sample
      * 1: Generated sample because of packet loss
      * 2-9: Flags for OffsetControl
      * \param offsetChangeIndex When did the offset change occured
      * \param timestamp The timestamp of this sample in ms (since 1/1/1970)
      */
     void _notifyData (int dataCh1, int dataCh2, int dataCh3, int dataCh4,
                       unsigned char flag, unsigned char offsetChangeIndex,
                       qint64 timestamp);

     /*!
      * It starts the thread that will call the read function and thus start
      * listening for incoming data.
      */
     void _startReadThread ();

     /*!
      * It stops the thread that handles the reading of incoming data
      */
     void _stopReadThread ();

     /*!
      * It sends the start command
      */
     bool _sendStartCommand ();
     /*!
      * Start sampling at the predefined sampling rate.
      *
      * \return True or false according to the success
      * or failure of the operation
      */
     bool _start ();

     /*!
      * Hold sampling (setting sampling rate to 0)
      *
      * \return True or false according to the success
      * or failure of the operation
      */
     bool _stop ();

     /*!
      * To Be Commented
      */
     bool _openDevice ();
};

#endif // READERENGINE_H
