#ifndef ENOBIODATA_H
#define ENOBIODATA_H

#include "EnobioAPI_global.h"

/*! \class EnobioData EnobioData.h
 *
 *  \brief This is the class encapsulating a sample of enobio data together with the required metadata
 *
 */

class ENOBIOAPISHARED_EXPORT EnobioData {
public:

    /*!
     * \enum EnobioData::flagValue
     * Enumeration with the possible values for the flag parameter
     */
    enum flagValue
    {
        FLAG_NORMAL_DATA = 0,
        FLAG_LOST_PACKET,
        FLAG_POSITIVE_OFFSET_CHANGE_CH1,
        FLAG_POSITIVE_OFFSET_CHANGE_CH2,
        FLAG_POSITIVE_OFFSET_CHANGE_CH3,
        FLAG_POSITIVE_OFFSET_CHANGE_CH4,
        FLAG_NEGATIVE_OFFSET_CHANGE_CH1,
        FLAG_NEGATIVE_OFFSET_CHANGE_CH2,
        FLAG_NEGATIVE_OFFSET_CHANGE_CH3,
        FLAG_NEGATIVE_OFFSET_CHANGE_CH4,
        FLAG_POSITIVE_OFFSET_CHANGE_CH1_ON_LOST_PACKET,
        FLAG_POSITIVE_OFFSET_CHANGE_CH2_ON_LOST_PACKET,
        FLAG_POSITIVE_OFFSET_CHANGE_CH3_ON_LOST_PACKET,
        FLAG_POSITIVE_OFFSET_CHANGE_CH4_ON_LOST_PACKET,
        FLAG_NEGATIVE_OFFSET_CHANGE_CH1_ON_LOST_PACKET,
        FLAG_NEGATIVE_OFFSET_CHANGE_CH2_ON_LOST_PACKET,
        FLAG_NEGATIVE_OFFSET_CHANGE_CH3_ON_LOST_PACKET,
        FLAG_NEGATIVE_OFFSET_CHANGE_CH4_ON_LOST_PACKET,
    };

    static const int NUMBER_OF_CHANNELS = 4;

    /*!
     * Constructor of an EnobioData object
     *
     * \param dataCh1 Channel 1 data value.
     * \param dataCh2 Channel 2 data value.
     * \param dataCh3 Channel 3 data value.
     * \param dataCh4 Channel 4 data value.
     * \param flag Flag that informs about the consideration that has to be
     * taken regarding the current Enobio data
     * \param offsetChangeIndex It indicates a change of offset that happened.
     * Its value corresponds to the number of samples before the current one
     * when the offset actually changed.
     * \param timestamp that marks the time of the sample within the stremaing.
     */
    EnobioData (int dataCh1, int dataCh2, int dataCh3, int dataCh4,
                flagValue flag, unsigned char offsetChangeIndex,
                qint64 timestamp);

    /*!
     * Get the value of channel 1.
     *
     * \return Channel 1 data.
     */
    int getChannel1 () const;

    /*!
     * To Be Commented
     */
    void setChannel1 (double data);

    /*!
     * Get the value of channel 2.
     *
     * \return Channel 2 data.
     */
    int getChannel2 () const;

    /*!
     * To Be Commented
     */
    void setChannel2 (double data);

    /*!
     * Get the value of channel 3
     */
    int getChannel3 () const;

    /*!
     * To Be Commented
     */
    void setChannel3 (double data);

    /*!
     * Get the value of channel 4
     */
    int getChannel4 () const;

    /*!
     * To Be Commented
     */
    void setChannel4 (double data);

    /*!
     * Get the value of the channel requested with the parameter.
     *
     * \param channel Number of the requested channel. It shall be a number from
     * 1 to 4.
     *
     * \return Value of the data from the requested channel
     */
    int getDataFromChannel (unsigned char channel) const;

    /*!
     * To be Commented
     */
    void setDataToChannel (double data, unsigned char channel);

    /*!
     * Get the flag of this Enobio data
     *
     * \return It returns a value from the flagValue enumeration.
     */
    flagValue getFlag () const;

    /*!
     * Get the number of samples before this one that the offset changed value.
     */
    unsigned char getOffsetChangeIndex () const;

    /*!
     * Get the timestamp of this sample
     */
    qint64 getTimestamp () const;

private:

    /*!
     * The value of channel 1
     */
    int _channel1;

    /*!
     * The value of channel 2
     */
    int _channel2;

    /*!
     * The value of channel 3
     */
    int _channel3;

    /*!
     * The value of channel 4
     */
    int _channel4;

    /*!
     * The flag
     */
    flagValue _flag;

    /*!
     * The offset change index
     */
    unsigned char _offsetChangeIndex;

    /*!
     * The timestamp
     */
    qint64 _timestamp;
};

#endif // ENOBIODATA_H

