#ifndef ENOBIOPRIVATE_H
#define ENOBIOPRIVATE_H

#include <string>

#include "FTDIDevice.h"
#include "SIPPProtocol.h"
#include "ReaderEngine.h"

/*!
 * \class EnobioPrivate
 *
 * \brief This is hidding the implementation code from external users that
 * access the methods and data members in the Enobio class.
 */
class EnobioPrivate
{
public:

    /*!
     * Defaul constructor of an Enobio object
     */
    EnobioPrivate ();

    /*!
     * Enobio destructor
     */
    ~EnobioPrivate ();

    /*!
     * It performs the operations required to initialize the hardware
     */
    bool openDevice ();

    /*!
     * It performs the operations required to close the hardware device
     */
    void closeDevice ();

    /*!
     * Start sampling at the predefined sampling rate.
     */
    void startStreaming ();

    /*!
     * Hold sampling (setting sampling rate to 0)
     */
    void stopStreaming ();

    /*!
     * It checks if the recorded device is present.
     */
    bool ping ();

    /*!
     * It returns the Property object identified with a property name.
     *
     * \param propertyName Name of the property to be obtained.
     *
     * \return Reference to the solicited property.
     */
    const Property* getProperty (std::string propertyName);

    /*!
     * It configures an API local property to be identical to the parameter.
     *
     * \param property Configuration property object.
     *
     * \return It returns true if the method succeeded on setting the property.
     * Otherwise a null object is returned.
     */
    bool setProperty(const Property& property);

    /*!
     * Access to the data producer in charge of populating the ECG/EEG data.
     *
     * \return Constant reference to the instance of the ECG/EEG streaming data
     * producer.
     */
    IDataProducer& getStreamingDataProducer ();

    /*!
     * Access to the data producer in charge of populating the status of the
     * ECG/EEG channels and the device itself.
     *
     * \return Constant reference to the instance of the status data producer.
     */
    IDataProducer& getStatusDataProducer ();

    /*!
     * It adds a DataConsumer to the list of consumers of the streaming
     * data producer of ReaderEngine
     *
     * \param dataConsumer The DataConsumer to add
     */
    void addStreamingDataConsumerEndPoint ( IDataConsumer& dataConsumer);

    /*!
     * It adds a DataConsumer to the list of consumers of the status
     * data producer of ReaderEngine
     *
     * \param dataConsumer The DataConsumer to add
     */
    void addStatusDataConsumerEndPoint ( IDataConsumer& dataConsumer);

    /*!
     * It removes a DataConsumer to the list of consumers of the streaming
     * data producer of ReaderEngine
     *
     * \param dataConsumer The DataConsumer to add
     */
    void removeStreamingDataConsumerEndPoint (const IDataConsumer& dataConsumer);

    /*!
     * It removes a DataConsumer to the list of consumers of the status
     * data producer of ReaderEngine
     *
     * \param dataConsumer The DataConsumer to add
     */
    void removeStatusDataConsumerEndPoint (const IDataConsumer& dataConsumer);


private:

    /*!
     * Instance of the HW device Enobio is using
     */
    FTDIDevice ftdiDevice;

    /*!
     * Instance of the communication protocol Enobio is using
     */
    SIPPProtocol sippProtocol;

    /*!
     * Instance of the EEG/ECG reader engine
     */
    ReaderEngine readerEngine;

//    /*!
//     * Set of Property objects to be set to the device
//     */
//    PropertyCollection propertyCollection;

    Property emptyProperty;
    Property localVersionProperty;
    Property remoteVersionProperty;
    Property localTransmissionChannelProperty;
    Property remoteTransmissionChannelProperty;
    Property channel1stateProperty;
    Property channel2stateProperty;
    Property channel3stateProperty;
    Property channel4stateProperty;
    Property baudRateProperty;
    Property wordLengthProperty;
    Property stopBitsProperty;
    Property parityProperty;
    Property readTimeoutProperty;
    Property latencyTimerProperty;


};

#endif // ENOBIOPRIVATE_H
