#ifndef SIPPPROTOCOL_H
#define SIPPPROTOCOL_H

#include "Protocol.h"

#define BYTES_PER_SAMPLE    8

/*!
 * \class SIPPProtocol SIPPProtocol.h
 *
 * \brief Implementation of the SIPP protocol. It implements the Enobio
 * datagram protocol too.
 */
class SIPPProtocol : public Protocol
{
public:

    /*!
     * \enum SIPPProtocol::packetLengths
     *
     * Length of the packets that can be generated by the class.
     */
    enum packetLengths
    {
        ENOBIO_READ_REQUEST_PACKET_LENGTH = 0x11,
        ENOBIO_READ_RESPONSE_PACKET_LENGTH = 0x07,
        ENOBIO_WRITE_REQUEST_PACKET_LENGTH = 0x13,
        SIPP_READ_REQUEST_PACKET_LENGTH = 0x06,
        SIPP_READ_RESPONSE_PACKET_LENGTH = 0x08,
        EEPROM_READ_REQUEST_PACKET_LENGTH = 0x07,
        EEPROM_READ_RESPONSE_PACKET_LENGTH = 0x07,
        EEPROM_WRITE_REQUEST_PACKET_LENGTH = 0x08,
        EEPROM_WRITE_RESPONSE_PACKET_LENGTH = 0x06,
        STREAMING_DATA_PACKET_LENGTH = 0x3B,
    };

    enum sippCommands
    {
        SIPP_CMD_REQUEST_ID = 0xB7,
    };


    /*!
     * Empty constructor
     */
    SIPPProtocol ();

    /*!
     * Empty destructor
     */
    ~SIPPProtocol ();

    /*!
     * This method implements a parsing state machine for incoming data.
     *
     * \param The byte to pass through the state machine
     * \return True when a sample is available and when a packet wass lost
     * false otherwise
     */
    virtual bool parseByte (unsigned char byte);

    /*!
     * Unused
     */
    virtual bool parsePacket (unsigned char * packet);

    /*!
     * Get the available data (in samples)
     * \return The sample byte array
     */
    virtual const unsigned char * getData ();

    /*!
     * Construct a SIPP packet with the basic header and the command.
     * The byte order is: 1.packet length, 2.Destination MSB, 3.Destination LSB,
     * 4.Source MSB, 5.Source LSB and 6.command
     * \param sippReceiver The receiving radio for this command
     * \param sippSender The sending radio for this command
     * \param cmd The SIPP command
     * \param bytes Output parameter. The function writes the SIPP packet in
     * the memory pointed by this parameter.
     */
    static void getSIPPReadPacket (unsigned short sippReceiver,
                                   unsigned short sippSender,
                                   unsigned char cmd, unsigned char * bytes);

    /*!
     * Construct a SIPP packet with the basic header, the command the EEPROM
     * address and the data to write.
     * The byte order is: 1.packet length, 2.Destination MSB, 3.Destination LSB,
     * 4.Source MSB, 5.Source LSB 6.command, 7. EEPROM address, 8.data
     * \param sippReceiver The receiving radio for this command
     * \param sippSender The sending radio for this command
     * \param address The EEPROM address to write to
     * \param data The value to be written in the EEPROM address
     * \param bytes Output parameter. The function writes the SIPP packet in
     * the memory pointed by this parameter.
     */
    static void getEEPROMWritePacket (unsigned short sippReceiver,
                                      unsigned short sippSender,
                                      unsigned char address, unsigned char data,
                                      unsigned char * bytes);

    /*!
     * Construct a SIPP packet with the basic header, the command and
     * the EEPROM address to read.
     * The byte order is: 1.packet length, 2.Destination MSB, 3.Destination LSB,
     * 4.Source MSB, 5.Source LSB 6.command, 7. EEPROM address
     * \param sippReceiver The receiving radio for this command
     * \param sippSender The sending radio for this command
     * \param address The EEPROM address to write to
     * \param bytes Output parameter. The function writes the SIPP packet in
     * the memory pointed by this parameter.
     */
    static void getEEPROMReadPacket (unsigned short sippReceiver,
                                     unsigned short sippSender,
                                     unsigned char address,
                                     unsigned char * bytes);

    /*!
     * This function builds a read-type SIPP packet with the provided values.
     *
     * \param sippReceiver The receiving radio for this command
     * \param sippSender The sending radio for this command
     * \param sourcePort Source port number of the SIPP packet.
     * \param sourceAddress Source address of the SIPP packet.
     * \param destPort Destination port number of the SIPP packet.
     * \param destAddres Destination address of the SIPP packet.
     * \param bytes Output parameter. The function writes the SIPP packet in
     * the memory pointed by this parameter.
     *
     * \remarks The caller shall allocate sufficient memory for holding the
     * SIPP packet that will be copied to the bytes variable. The
     * SIPPProtocol::packetLengths enumeration can be used in order to know how
     * many bytes are needed for each different SIPP packet.
     */
    static void getEnobioReadPacket (unsigned short sippReceiver,
                                     unsigned short sippSender,
                                     unsigned int sourcePort,
                                     unsigned int sourceAddress,
                                     unsigned int destPort,
                                     unsigned int destAddress,
                                     unsigned char * bytes);

    /*!
     * This function builds a write-type SIPP packet with the provided values.
     *
     * \param sippReceiver The receiving radio for this command
     * \param sippSender The sending radio for this command
     * \param sourcePort Source port number of the SIPP packet.
     * \param sourceAddress Source address of the SIPP packet.
     * \param destPort Destination port number of the SIPP packet.
     * \param destAddres Destination address of the SIPP packet.
     * \param payload Payload of the SIPP packet
     * \param bytes Output parameter. The function writes the SIPP packet in
     * the memory pointed by this parameter.
     *
     * \remarks The caller shall allocate sufficient memory for holding the
     * SIPP packet that will be copied to the bytes variable. The
     * SIPPProtocol::packetLengths enumeration can be used in order to know how
     * many bytes are needed for each different SIPP packet.
     */
    static void getEnobioWritePacket (unsigned short sippReceiver,
                                      unsigned short sippSender,
                                      unsigned int sourcePort,
                                      unsigned int sourceAddress,
                                      unsigned int destPort,
                                      unsigned int destAddress,
                                      unsigned int payload,
                                      unsigned char * bytes);


    /*!
     * This returns the number of lost packet that have been detected after
     * parsing a new received SIPP packet.
     *
     * \return Number of lost packets. The maximum number of lost packet that
     * the protocol can detect is 255.
     */
    int getNumberOfLostPackets ();

    /*!
     * This function returns the Enobio ID that is needed for the wireless
     * transmission of the data.
     *
     * \return Value of the Enobio ID which is used in the SIPP protocol to
     * communicate with the Enobio device.
     */
    unsigned short getEnobioID ();

    /*!
     * This function returns the USB ID that is needed for the wireless
     * transmission of the data.
     *
     * \return Value of the USB ID wich is used in the SIPP protocol to
     * communicate with the Enobio device.
     */
    unsigned short getUSBID ();

    /*!
     * To Be Commented
     */
    void resetUSBID ();

    /*!
     * This function returns the channel of transmission over which the SIPP
     * protocol packets are sent.
     *
     * \return Number of the channel from 0 to 15.
     */
    unsigned char getEnobioTransmissionChannel ();


    /*!
     * This function sets the channel of transmission over which the SIPP
     * protocol packets are sent.
     *
     */
    void setEnobioTransmissionChannel (unsigned char channel);


    /*!
     * This function returns the channel of transmission over which the SIPP
     * protocol packets are sent.
     *
     * \return Number of the channel from 0 to 15.
     */
    unsigned char getUSBTransmissionChannel ();


    /*!
     * This function sets the channel of transmission over which the SIPP
     * protocol packets are sent.
     *
     */
    void setUSBTransmissionChannel (unsigned char channel);


    /*!
     * This function returns the state of the variable _eepromAck
     *
     * \return True if an aknowledgement has been received and false
     * if it is still expected.
     *
     */

    bool getEepromAck ();


    /*!
     * This function sets the state of variable _eepromAck in order to
     * check whether a write to the EEPROM has been succesful or not.
     *
     */
    void setEepromAck (bool state);


    /*!
     * This function returns the version of the firmware.
     *
     * \return Firmware version number.
     */
    unsigned char getUSBFWVersion ();

    /*!
     * This function returns the version of the firmware.
     *
     * \return Firmware version number.
     */
    unsigned char getEnobioFWVersion ();

    /*!
     * This sets a local variable which is used to maintain the integrity of
     * retrieving the version for the USB.
     *
     * \param usbFWVersion The value to set the local variable to
     */
    void setUSBFWVersion (unsigned char usbFWVersion);

    /*!
     * This sets a local variable which is used to maintain the integrity of
     * retrieving the version for the Enobio.
     *
     * \param enobioFWVersion The value to set the local variable to
     */
    void setEnobioFWVersion (unsigned char enobioFWVersion);

    /*!
     * This function returns the counter of last received packet.
     *
     * \return Counter of the last received packet.
     */
    int getPacketCounter ();

    /*!
     * This function sets the counter . Used mainly to reset the variable
     * when the streaming is started or stopped.
     *
     */
    void setPacketCounter (int packetCounter);

    /*!
     * This function returns whether an offset change has happened or not.
     *
     * \return true if an offset change has happened, false otherwise.
     */
    bool isOffsetChanged ();

    /*!
     * This function returns the number of the packet where the offset change
     * occured.
     *
     * \return number of the packet where the last offset change happened.
     */
    int getOffsetCounter ();

    /*!
     * \property SIPPProtocol::SAMPLES_PER_PACKET
     *
     * The number of samples per packet.
     */
    static const unsigned char SAMPLES_PER_PACKET;

    /*!
     * This function resets the values of some variables that are used
     * for the parsing of the incoming data
     *
     */
    void reset();


private:
    /*!
     * The ID of Enobio used to construct the SIPP packet
     */
    unsigned short _enobioID;

    /*!
     * The ID of the USB receiver used to construct the SIPP packet
     */
    unsigned short _usbID;

    /*!
     * The current state of the state machine on the SIPP level
     */
    unsigned char _sippState;

    /*!
     * The current state of the state machine on the Enobio datagram level
     */
    unsigned char _enobioState;

    /*!
     * A counter of the incoming bytes of a packet
     */
    unsigned short _byteCounter;

    /*!
     * The index of the enobio data buffer
     */
    unsigned short _enobioIndex;

    /*!
     * This is a temporary variable to hold the MSB of an integer so that it
     * can be reproduced on the arrival of the LSB.
     */
    unsigned char _lsbByte;

    /*!
     * The channel of the tranceiver
     */
    unsigned char _usbTransmissionChannel;

    /*!
     * The channel of the tranceiver
     */
    unsigned char _enobioTransmissionChannel;

    /*!
     * This variable is used to track the reception of acknowledges of
     * write EEPROM commands
     */
    bool _eepromAck;

    /*!
     * The firmware version of Enobio
     */
    unsigned char _enobioFWVersion;

    /*!
     * The firmware version of the USB receiver
     */
    unsigned char _usbFWVersion;

    /*!
     * This is a flag to indicate which where this SIPP packet came from,
     * Enobio or te USB device.
     */
    bool _getLocalVariable;

    /*!
     * This is the packetCounter to monitor packet loss.
     * This is a 0-255 number included in the SIPP packet
     */
    int _offsetCounter;

    /*!
     * This is the packetCounter to monitor packet loss.
     */
    short _packetCounter;

    /*!
     * How many packets were lost and thus need to be filled in.
     */
    int _fillPackets;

    /*!
     * This variable indicates if an offset change occured.
     * It switches from true to false and vice versa.
     */
    bool _toggle;

    /*!
     * This keeps the toggle of the penultimate packet,
     * in order to check for changes with the current one
     */
    bool _previousToggle;

    /*!
     * Array to keep the data extracted from the starlab datagram.
     */
    unsigned char _enobioData[BYTES_PER_SAMPLE];

    /*!
     * Array to keep the last complete data for a channel
     */
    unsigned char _completeData[BYTES_PER_SAMPLE];

    /*!
     * This function extracts the data of a SIPP packet.
     * It returns true when the data of a whole packet has been extracted
     * and false if there are not yet enough samples.
     */
    bool _parseStarlabDatagram(unsigned char byte);

    /*!
     * This function checks whether the lenght of the sipp packet passed as
     * parameter is a valid one.
     *
     * \param byte Value that indicates the length of a SIPP packet.
     *
     * \return true if the lenght is a valid one, false otherwise.
     */
    bool _isValidSIPPPacketLength (unsigned char byte);

};

#endif // SIPPPROTOCOL_H
