#ifndef ENOBIO_H
#define ENOBIO_H

#include <string>

#include "EnobioAPI_global.h"
#include "../interfaces/IDataProcessor.h"
#include "../interfaces/IDataProducer.h"
#include "Property.h"
#include "../interfaces/IDataProcessor.h"


/*! \class Enobio Enobio.h
 *
 *  \brief This is the class that allows access to the Enobio data streaming.
 *
 * This class holds just the external interface of the API. The implementation
 * of the code is done in StarlabPrivate
 *
 * It inheritates from the base class DataProcessor so it can interchange data
 * with other peers.
 *
 */

class ENOBIOAPISHARED_EXPORT Enobio : public IDataProcessor {
public:

    /*!
     * The strings to create the properties of ENOBIO
     */
    static const std::string STR_PROPERTY_USB_FIRMWARE_VERSION;
    static const std::string STR_PROPERTY_ENOBIO_FIRMWARE_VERSION;
    static const std::string STR_PROPERTY_USB_TRANSMISSION_CHANNEL;
    static const std::string STR_PROPERTY_ENOBIO_TRANSMISSION_CHANNEL;
    static const std::string STR_PROPERTY_ENABLE_CHANNEL_1;
    static const std::string STR_PROPERTY_ENABLE_CHANNEL_2;
    static const std::string STR_PROPERTY_ENABLE_CHANNEL_3;
    static const std::string STR_PROPERTY_ENABLE_CHANNEL_4;
    static const std::string STR_PROPERTY_BAUD_RATE;
    static const std::string STR_PROPERTY_WORD_LENGTH;
    static const std::string STR_PROPERTY_STOP_BITS;
    static const std::string STR_PROPERTY_PARITY;
    static const std::string STR_PROPERTY_READ_TIMEOUT;
    static const std::string STR_PROPERTY_LATENCY_TIMER;
    static const std::string ENOBIO_API_VERSION;


    /*!
     * Constructor of an Enobio object
     */
    Enobio ();

    /*!
     * Enobio destructor
     */
    virtual ~Enobio ();


    /*!
     * The ids of the available producers.
     */
    enum producerIDs
    {
        ENOBIO_DATA=1,
        STATUS=2
    };           

    /*!
     * It performs the operations required to initialize the hardware
     */
    bool openDevice ();

    /*!
     * It performs the operations required to close the hardware device
     */
    void closeDevice ();

    /*!
     * Start sampling at the predefined sampling rate.
     */
    void startStreaming ();

    /*!
     * Hold sampling (setting sampling rate to 0)
     */
    void stopStreaming ();

    /*!
     * Checks whether the Enobio is switched on.
     */
    bool ping ();

    /*!
     * It returns the Property object identified with a property name.
     *
     * \param propertyName Name of the property to be obtained.
     *
     * \return Reference to the solicited property.
     */
    const Property* getProperty (std::string propertyName);

    /*!
     * It configures an API property to be identical to the parameter.
     *
     * \param property Configuration property object.
     *
     * \return It returns true if the method succeeded on setting the property.
     * Otherwise a null object is returned.
     */
    bool setProperty(const Property& property);


    /*!
     * It returns the DataConsumer object identified with an ID.
     *
     * \param id The identification of the DataConsumer to be obtained.
     *
     * \return Reference to the solicited DataConsumer.
     */

    virtual IDataConsumer * getDataConsumer (const int id);

    /*!
     * It returns the DataProducer object identified with an ID.
     *
     * \param id The identification of the DataProducer to be obtained.
     *
     * \return Reference to the solicited DataProducer.
     */

    virtual IDataProducer * getDataProducer (const int id);


    /*!
     * It adds a DataConsumer to the list of the consumers of the Producer
     * identified by the parameter id
     *
     * \param id Identification of the Producer to use
     * \param dataConsumer The DataConsumer to add to the Producer
     *
     * \return If the producer specified does not exist the function returns false
     *
     */

    virtual bool registerConsumer (const int id,
                                    IDataConsumer& dataConsumer);

    /*!
     * It removes a DataConsumer from the list of consumers of the
     * Producer identified by the parameter id
     *
     * \param id Identification of the Producer to use
     * \param dataConsumer The DataConsumer to remove from the Producer
     *
     * \return If the producer specified does not exist the function returns false
     *
     */

    virtual bool deregisterConsumer (const int id,
                                     const IDataConsumer& dataConsumer);


private:

    /*!
     * Instance of the implementation class.
     */
    void * _enobioPrivate;


};

#endif // ENOBIO_H
